# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import logging
import hashlib
import hmac
import urllib
import requests

# Set version for API
from .exception import KhipuError

VERSION_KHIPU_SERVICE = '2.0'


class Transaction(object):
    def __init__(self, method, service, data, params):
        self.method = method
        self.service = service
        self.data = data
        self.params = params
        self.headers = ""


class KhipuService(object):
    def __init__(self, receiver_id, secret):
        self.receiver_id = receiver_id
        self.secret = secret

        # Services
        self.CREATE_PAYMENT = 'payments'
        self.GET_PAYMENT_TOKEN = 'payments'
        self.GET_PAYMENT_ID = 'payments/{id}'
        self.REFUND_PAYMENT = '/payments/{id}/refunds'
        self.DELETE_PAYMENT = ''
        self.CREATE_RECEIVER = ''

        # Set url to API
        self.API_URL = 'https://khipu.com/api/%s/' % VERSION_KHIPU_SERVICE

        # Declare status error
        self.STATUS_ERRORS = {
            400: 'Invalid Data.',
            403: 'Authorization Error.',
            503: 'Operation Error.'
        }

        # Init logger
        self.logger = logging.getLogger(__name__)
        logging.basicConfig(level=logging.INFO)
        logging.getLogger("tbk.service").setLevel(logging.DEBUG)

    def request(self, method, service, data, params=None):
        transaction = Transaction(method=method, service=self.API_URL + service, data=data, params=params)
        headers = self.create_headers(transaction)
        transaction.headers = headers
        response = self.do_request(transaction)
        self.validate_response(response)
        return response.json()

    def validate_response(self, response):
        if response.status_code in self.STATUS_ERRORS.keys():
            raise KhipuError(
                message="Status Code {0}: {1}".format(response.status_code, self.STATUS_ERRORS[response.status_code]),
                error_code=response.status_code)

    def create_headers(self, transaction):
        # Set headers
        headers = {
            'Authorization': '%s:%s' % (self.receiver_id, self.do_hash(transaction),),
            'Content-Type': 'application/x-www-form-urlencoded'
        }

        return headers

    def do_request(self, transaction):

        try:
            return requests.request(transaction.method, transaction.service, params=transaction.params,
                                    data=transaction.data, headers=transaction.headers)
        except Exception as e:
            self.logger.error(e.message)
            raise KhipuError(message=e.message, error_code=500)

    def do_hash(self, transaction):
        """
        Generate hash specific for Khipu
        """
        return hmac.new(self.secret.encode(),
                        self.create_concatenated_data(transaction).encode(),
                        hashlib.sha256).hexdigest()

    @classmethod
    def create_concatenated_data(cls, transaction):
        # Init vars
        all_data = {}
        cad = '%s&%s' % (transaction.method, cls.__url_encode(transaction.service))

        if transaction.params is not None:
            for key, value in transaction.params.iteritems():
                all_data[key] = value

        if transaction.data is not None:
            for key, value in transaction.data.iteritems():
                all_data[key] = value

        # Remove amount of all_data and set in cad
        if 'amount' in all_data:
            cad += '&{0}={1}'.format('amount', all_data['amount'])
            all_data.pop('amount', None)

        if all_data is not None or len(all_data) > 0:
            for key, value in iter(sorted(all_data.iteritems())):
                value = cls.__url_encode(value)
                cad += '&{0}={1}'.format(key, value)

        return cad

    @classmethod
    def __url_encode(cls, s):
        if s is None:
            return ''
        return urllib.quote_plus(s).replace("+", "%20").replace("*", "%2A").replace("%7E", "~")



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from setuptools import setup, find_packages

import os
import io

# Package meta-data.
NAME = 'cs-khipu'
DESCRIPTION = 'Unofficial Khipu Services Python SDK.'
URL = 'https://github.com/cornershop/python-khipu'
EMAIL = 'tech@cornershopapp.com'
AUTHOR = 'Cornershop'

# What packages are required for this module to be executed?
REQUIRED = [
    "requests", "arrow"
]

here = os.path.abspath(os.path.dirname(__file__))

# Import the README and use it as the long-description.
# Note: this will only work if 'README.rst' is present in your MANIFEST.in file!
with io.open(os.path.join(here, 'README.rst'), encoding='utf-8') as f:
    long_description = '\n' + f.read()

# Load the package's __version__.py module as a dictionary.
about = {}
with open(os.path.join(here, "khipu", '__about__.py')) as f:
    exec (f.read(), about)

setup(name=NAME,
      version=str(about['__version__']),
      description=DESCRIPTION,
      long_description=long_description,
      url=URL,
      author=AUTHOR,
      author_email=EMAIL,
      license='MIT',
      install_requires=REQUIRED,
      packages=find_packages(exclude=('tests',)),
      zip_safe=False)



# -*- coding: utf-8 -*-
from collections import OrderedDict

from .service import KhipuService

from .exception import KhipuError


class KhipuApi(KhipuService):
    def create_payment(self, **kwargs):
        required = frozenset(['subject', 'currency', 'amount'])
        optionals = [
            'transaction_id',
            'custom',
            'body',
            'bank_id',
            'return_url',
            'cancel_url',
            'picture_url',
            'notify_url',
            'notify_api_version',
            'expires_date',
            'send_email',
            'payer_name',
            'payer_email',
            'send_reminders',
            'responsible_user_email',
            'fixed_payer_personal_identifier',
            'integrator_fee'
        ]

        if not required.issubset(frozenset(kwargs.keys())):
            raise KhipuError(message="Create Payment should have this data: %s" % ', '.join(required),
                             error_code=500)

        data = {
            'subject': kwargs.pop('subject'),
            'currency': kwargs.pop('currency'),
            'amount': kwargs.pop('amount').replace('$', '')
        }

        for v in optionals:
            if v in kwargs:
                valor = kwargs.pop(v)
                if valor is not None:
                    data[v] = valor

        return self.request('POST', self.CREATE_PAYMENT, data)

    def get_payment_by_token(self, notification_token):
        if notification_token is None:
            raise KhipuError(message="Get Payment by Token should have this data: notification token",
                             error_code=500)

        params = {
            'notification_token': notification_token,
        }

        return self.request('GET', self.GET_PAYMENT_TOKEN, data=None, params=params)

    def get_payment_id(self, payment_id):
        if payment_id is None or payment_id == "":
            raise KhipuError(message="Invalid PaymentID", error_code=500)

        return self.request('GET', self.GET_PAYMENT_ID.format(id=payment_id), data=None, params=None)

    def refund_payment(self, payment_id):
        if payment_id is None or payment_id == "":
            raise KhipuError(message="Invalid PaymentID", error_code=500)

        return self.request('POST', self.REFUND_PAYMENT.format(id=payment_id), data=None, params=None)
